/*
 * j_setloc.c
 *
 * By Ross Ridge
 * Public Domain
 *
 * Implementation of jis_set_locale_ctype()
 *
 */

#include <locale.h>

#include "defs.h"

#ifdef CONFIG_SCCS_IDS
static char const sccs_id[] = "@(#) myjis j_setloc.c 1.5 98/06/01 22:28:11";
#endif

enum jis_locale_ctypes __jis_locale_ctype = CONFIG_JIS_LOCALE_C;

enum jis_locale_ctypes
jis_set_locale_ctype(enum jis_locale_ctypes loc) {
	if (loc == JIS_LOCALE_DEFAULT) {
		char const *s = setlocale(LC_CTYPE, NULL);
		assert(s != NULL);
		/*
		 * If the locale returned by setlocale "C" then it may
		 * have not recognized the user's choice of a Japanese
		 * locale.  In that case we try to figure the locale
		 * using POSIX locale environment variables.
		 */
		if (s[0] == 'C' && s[1] == '\0') {
			s = getenv("LC_ALL");
			if (s == NULL) {
				s = getenv("LC_CTYPE");
				if (s == NULL) {
					s = getenv("LANG");
					if (s == NULL) {
						s = "";
					}
				}
			}
		}

		loc = CONFIG_JIS_LOCALE_C;
		if (s[0] == 'J' && s[1] == 'a') {
			loc = JIS_LOCALE_SHIFT_JIS;
		} else if (s[0] == 'j') {
			if (s[1] == 'a') {
				loc = JIS_LOCALE_EUC;
			} else if (s[1] == 'i' && s[2] == 's') {
				loc = JIS_LOCALE_JIS;
			}
#ifdef __JIS_CONFIG_HAVE_UNICODE
		} else if (strcmp(s, "utf8") == 0) {
			loc = JIS_LOCALE_UTF8;
#endif
		}

#if 0
		printf("@@@: s = %s, loc = %d\n", s, (int) loc);
#endif
	}
	if (loc != JIS_LOCALE_QUERY) {
		__jis_locale_ctype = loc;
	}
	return __jis_locale_ctype;
}
