/*
 * kana.h
 *
 * By Ross Ridge
 * Public Domain
 *
 * @(#) myjis kana.h 1.10 98/06/01 22:28:18
 *
 */

#ifndef __JIS_KANA_H__
#define __JIS_KANA_H__

#include "wide.h"
#include "config-sh.h"


wide_char *wide_to_hiragana_str(wide_char *ws);
wide_char *wide_to_katakana_str(wide_char *ws);
wide_int wide_to_katakana_vowel(wide_int wc);
wide_char *wide_to_hiragana_vowel_str(wide_char *ws);
wide_char *wide_to_katakana_vowel_str(wide_char *ws);

#define __JIS_R2K_MAX_ROMAN_LEN	4
#define __JIS_R2K_MAX_KANA_LEN	3

struct jis_r2k_kana {
	char roman[__JIS_R2K_MAX_ROMAN_LEN + 1];
	wide_char kana[__JIS_R2K_MAX_KANA_LEN + 1];
	int flags;
};

int wide_roman_to_kana(struct jis_r2k_kana const **kana, size_t len,
		       wide_char const *ws, int flags);
int wide_roman_to_kana_str(wide_char *wd, size_t *plen, wide_char const **pws,
			   int flags);
int wide_roman_to_kana_list(wide_char **list, size_t list_len, 
			    wide_char const *src, int flags, int *err);
int wide_kana_to_roman(struct jis_r2k_kana const **roman,
		       wide_char const *ws, int flags);
int wide_kana_to_roman_str(wide_char *wd, size_t *plen, wide_char const **pws,
			   int flags);

#define JIS_R2K_HEPBURN		00001
#define JIS_R2K_KUNREI		00002
#define JIS_R2K_NIPPON		00004
#define JIS_R2K_ALL		00007
#define JIS_R2K_KATAKANA	00010
#define JIS_R2K_PARTICLE	00020
#define JIS_R2K_OBSOLETE	00040
#define JIS_R2K_EXTENDED	00100

#define JIS_R2K_ERR_UNKNOWN	1
#define JIS_R2K_ERR_AMBIGUOUS	2


#ifdef __JIS_CONFIG_USE_UNICODE
#define __JIS_HIRAGANA_BASE	((wide_char) 0x3040)
#define __JIS_KATAKANA_BASE	((wide_char) 0x30A0)
#define __JIS_HALF_KATAKANA_BASE ((wide_char) 0xFF60)
#else
#define __JIS_HIRAGANA_BASE	((wide_char) 0xA4A0)
#define __JIS_KATAKANA_BASE	((wide_char) 0xA5A0)
#define __JIS_HALF_KATAKANA_BASE ((wide_char) 0x00A0)
#endif

#define H_a	((wide_char) (__JIS_HIRAGANA_BASE + 0x1))
#define H_A	((wide_char) (__JIS_HIRAGANA_BASE + 0x2))
#define H_i	((wide_char) (__JIS_HIRAGANA_BASE + 0x3))
#define H_I	((wide_char) (__JIS_HIRAGANA_BASE + 0x4))
#define H_u	((wide_char) (__JIS_HIRAGANA_BASE + 0x5))
#define H_U	((wide_char) (__JIS_HIRAGANA_BASE + 0x6))
#define H_e	((wide_char) (__JIS_HIRAGANA_BASE + 0x7))
#define H_E	((wide_char) (__JIS_HIRAGANA_BASE + 0x8))
#define H_o	((wide_char) (__JIS_HIRAGANA_BASE + 0x9))
#define H_O	((wide_char) (__JIS_HIRAGANA_BASE + 0xA))
#define H_KA	((wide_char) (__JIS_HIRAGANA_BASE + 0xB))
#define H_GA	((wide_char) (__JIS_HIRAGANA_BASE + 0xC))
#define H_KI	((wide_char) (__JIS_HIRAGANA_BASE + 0xD))
#define H_GI	((wide_char) (__JIS_HIRAGANA_BASE + 0xE))
#define H_KU	((wide_char) (__JIS_HIRAGANA_BASE + 0xF))
#define H_GU	((wide_char) (__JIS_HIRAGANA_BASE + 0x10))
#define H_KE	((wide_char) (__JIS_HIRAGANA_BASE + 0x11))
#define H_GE	((wide_char) (__JIS_HIRAGANA_BASE + 0x12))
#define H_KO	((wide_char) (__JIS_HIRAGANA_BASE + 0x13))
#define H_GO	((wide_char) (__JIS_HIRAGANA_BASE + 0x14))
#define H_SA	((wide_char) (__JIS_HIRAGANA_BASE + 0x15))
#define H_ZA	((wide_char) (__JIS_HIRAGANA_BASE + 0x16))
#define H_SI	((wide_char) (__JIS_HIRAGANA_BASE + 0x17))
#define H_ZI	((wide_char) (__JIS_HIRAGANA_BASE + 0x18))
#define H_SU	((wide_char) (__JIS_HIRAGANA_BASE + 0x19))
#define H_ZU	((wide_char) (__JIS_HIRAGANA_BASE + 0x1A))
#define H_SE	((wide_char) (__JIS_HIRAGANA_BASE + 0x1B))
#define H_ZE	((wide_char) (__JIS_HIRAGANA_BASE + 0x1C))
#define H_SO	((wide_char) (__JIS_HIRAGANA_BASE + 0x1D))
#define H_ZO	((wide_char) (__JIS_HIRAGANA_BASE + 0x1E))
#define H_TA	((wide_char) (__JIS_HIRAGANA_BASE + 0x1F))
#define H_DA	((wide_char) (__JIS_HIRAGANA_BASE + 0x20))
#define H_TI	((wide_char) (__JIS_HIRAGANA_BASE + 0x21))
#define H_DI	((wide_char) (__JIS_HIRAGANA_BASE + 0x22))
#define H_tu	((wide_char) (__JIS_HIRAGANA_BASE + 0x23))
#define H_TU	((wide_char) (__JIS_HIRAGANA_BASE + 0x24))
#define H_DU	((wide_char) (__JIS_HIRAGANA_BASE + 0x25))
#define H_TE	((wide_char) (__JIS_HIRAGANA_BASE + 0x26))
#define H_DE	((wide_char) (__JIS_HIRAGANA_BASE + 0x27))
#define H_TO	((wide_char) (__JIS_HIRAGANA_BASE + 0x28))
#define H_DO	((wide_char) (__JIS_HIRAGANA_BASE + 0x29))
#define H_NA	((wide_char) (__JIS_HIRAGANA_BASE + 0x2A))
#define H_NI	((wide_char) (__JIS_HIRAGANA_BASE + 0x2B))
#define H_NU	((wide_char) (__JIS_HIRAGANA_BASE + 0x2C))
#define H_NE	((wide_char) (__JIS_HIRAGANA_BASE + 0x2D))
#define H_NO	((wide_char) (__JIS_HIRAGANA_BASE + 0x2E))
#define H_HA	((wide_char) (__JIS_HIRAGANA_BASE + 0x2F))
#define H_BA	((wide_char) (__JIS_HIRAGANA_BASE + 0x30))
#define H_PA	((wide_char) (__JIS_HIRAGANA_BASE + 0x31))
#define H_HI	((wide_char) (__JIS_HIRAGANA_BASE + 0x32))
#define H_BI	((wide_char) (__JIS_HIRAGANA_BASE + 0x33))
#define H_PI	((wide_char) (__JIS_HIRAGANA_BASE + 0x34))
#define H_HU	((wide_char) (__JIS_HIRAGANA_BASE + 0x35))
#define H_BU	((wide_char) (__JIS_HIRAGANA_BASE + 0x36))
#define H_PU	((wide_char) (__JIS_HIRAGANA_BASE + 0x37))
#define H_HE	((wide_char) (__JIS_HIRAGANA_BASE + 0x38))
#define H_BE	((wide_char) (__JIS_HIRAGANA_BASE + 0x39))
#define H_PE	((wide_char) (__JIS_HIRAGANA_BASE + 0x3A))
#define H_HO	((wide_char) (__JIS_HIRAGANA_BASE + 0x3B))
#define H_BO	((wide_char) (__JIS_HIRAGANA_BASE + 0x3C))
#define H_PO	((wide_char) (__JIS_HIRAGANA_BASE + 0x3D))
#define H_MA	((wide_char) (__JIS_HIRAGANA_BASE + 0x3E))
#define H_MI	((wide_char) (__JIS_HIRAGANA_BASE + 0x3F))
#define H_MU	((wide_char) (__JIS_HIRAGANA_BASE + 0x40))
#define H_ME	((wide_char) (__JIS_HIRAGANA_BASE + 0x41))
#define H_MO	((wide_char) (__JIS_HIRAGANA_BASE + 0x42))
#define H_ya	((wide_char) (__JIS_HIRAGANA_BASE + 0x43))
#define H_YA	((wide_char) (__JIS_HIRAGANA_BASE + 0x44))
#define H_yu	((wide_char) (__JIS_HIRAGANA_BASE + 0x45))
#define H_YU	((wide_char) (__JIS_HIRAGANA_BASE + 0x46))
#define H_yo	((wide_char) (__JIS_HIRAGANA_BASE + 0x47))
#define H_YO	((wide_char) (__JIS_HIRAGANA_BASE + 0x48))
#define H_RA	((wide_char) (__JIS_HIRAGANA_BASE + 0x49))
#define H_RI	((wide_char) (__JIS_HIRAGANA_BASE + 0x4A))
#define H_RU	((wide_char) (__JIS_HIRAGANA_BASE + 0x4B))
#define H_RE	((wide_char) (__JIS_HIRAGANA_BASE + 0x4C))
#define H_RO	((wide_char) (__JIS_HIRAGANA_BASE + 0x4D))
#define H_wa	((wide_char) (__JIS_HIRAGANA_BASE + 0x4E))
#define H_WA	((wide_char) (__JIS_HIRAGANA_BASE + 0x4F))
#define H_WI	((wide_char) (__JIS_HIRAGANA_BASE + 0x50))
#define H_WE	((wide_char) (__JIS_HIRAGANA_BASE + 0x51))
#define H_WO	((wide_char) (__JIS_HIRAGANA_BASE + 0x52))
#define H_N	((wide_char) (__JIS_HIRAGANA_BASE + 0x53))
#define K_a	((wide_char) (__JIS_KATAKANA_BASE + 0x1))
#define K_A	((wide_char) (__JIS_KATAKANA_BASE + 0x2))
#define K_i	((wide_char) (__JIS_KATAKANA_BASE + 0x3))
#define K_I	((wide_char) (__JIS_KATAKANA_BASE + 0x4))
#define K_u	((wide_char) (__JIS_KATAKANA_BASE + 0x5))
#define K_U	((wide_char) (__JIS_KATAKANA_BASE + 0x6))
#define K_e	((wide_char) (__JIS_KATAKANA_BASE + 0x7))
#define K_E	((wide_char) (__JIS_KATAKANA_BASE + 0x8))
#define K_o	((wide_char) (__JIS_KATAKANA_BASE + 0x9))
#define K_O	((wide_char) (__JIS_KATAKANA_BASE + 0xA))
#define K_KA	((wide_char) (__JIS_KATAKANA_BASE + 0xB))
#define K_GA	((wide_char) (__JIS_KATAKANA_BASE + 0xC))
#define K_KI	((wide_char) (__JIS_KATAKANA_BASE + 0xD))
#define K_GI	((wide_char) (__JIS_KATAKANA_BASE + 0xE))
#define K_KU	((wide_char) (__JIS_KATAKANA_BASE + 0xF))
#define K_GU	((wide_char) (__JIS_KATAKANA_BASE + 0x10))
#define K_KE	((wide_char) (__JIS_KATAKANA_BASE + 0x11))
#define K_GE	((wide_char) (__JIS_KATAKANA_BASE + 0x12))
#define K_KO	((wide_char) (__JIS_KATAKANA_BASE + 0x13))
#define K_GO	((wide_char) (__JIS_KATAKANA_BASE + 0x14))
#define K_SA	((wide_char) (__JIS_KATAKANA_BASE + 0x15))
#define K_ZA	((wide_char) (__JIS_KATAKANA_BASE + 0x16))
#define K_SI	((wide_char) (__JIS_KATAKANA_BASE + 0x17))
#define K_ZI	((wide_char) (__JIS_KATAKANA_BASE + 0x18))
#define K_SU	((wide_char) (__JIS_KATAKANA_BASE + 0x19))
#define K_ZU	((wide_char) (__JIS_KATAKANA_BASE + 0x1A))
#define K_SE	((wide_char) (__JIS_KATAKANA_BASE + 0x1B))
#define K_ZE	((wide_char) (__JIS_KATAKANA_BASE + 0x1C))
#define K_SO	((wide_char) (__JIS_KATAKANA_BASE + 0x1D))
#define K_ZO	((wide_char) (__JIS_KATAKANA_BASE + 0x1E))
#define K_TA	((wide_char) (__JIS_KATAKANA_BASE + 0x1F))
#define K_DA	((wide_char) (__JIS_KATAKANA_BASE + 0x20))
#define K_TI	((wide_char) (__JIS_KATAKANA_BASE + 0x21))
#define K_DI	((wide_char) (__JIS_KATAKANA_BASE + 0x22))
#define K_tu	((wide_char) (__JIS_KATAKANA_BASE + 0x23))
#define K_TU	((wide_char) (__JIS_KATAKANA_BASE + 0x24))
#define K_DU	((wide_char) (__JIS_KATAKANA_BASE + 0x25))
#define K_TE	((wide_char) (__JIS_KATAKANA_BASE + 0x26))
#define K_DE	((wide_char) (__JIS_KATAKANA_BASE + 0x27))
#define K_TO	((wide_char) (__JIS_KATAKANA_BASE + 0x28))
#define K_DO	((wide_char) (__JIS_KATAKANA_BASE + 0x29))
#define K_NA	((wide_char) (__JIS_KATAKANA_BASE + 0x2A))
#define K_NI	((wide_char) (__JIS_KATAKANA_BASE + 0x2B))
#define K_NU	((wide_char) (__JIS_KATAKANA_BASE + 0x2C))
#define K_NE	((wide_char) (__JIS_KATAKANA_BASE + 0x2D))
#define K_NO	((wide_char) (__JIS_KATAKANA_BASE + 0x2E))
#define K_HA	((wide_char) (__JIS_KATAKANA_BASE + 0x2F))
#define K_BA	((wide_char) (__JIS_KATAKANA_BASE + 0x30))
#define K_PA	((wide_char) (__JIS_KATAKANA_BASE + 0x31))
#define K_HI	((wide_char) (__JIS_KATAKANA_BASE + 0x32))
#define K_BI	((wide_char) (__JIS_KATAKANA_BASE + 0x33))
#define K_PI	((wide_char) (__JIS_KATAKANA_BASE + 0x34))
#define K_HU	((wide_char) (__JIS_KATAKANA_BASE + 0x35))
#define K_BU	((wide_char) (__JIS_KATAKANA_BASE + 0x36))
#define K_PU	((wide_char) (__JIS_KATAKANA_BASE + 0x37))
#define K_HE	((wide_char) (__JIS_KATAKANA_BASE + 0x38))
#define K_BE	((wide_char) (__JIS_KATAKANA_BASE + 0x39))
#define K_PE	((wide_char) (__JIS_KATAKANA_BASE + 0x3A))
#define K_HO	((wide_char) (__JIS_KATAKANA_BASE + 0x3B))
#define K_BO	((wide_char) (__JIS_KATAKANA_BASE + 0x3C))
#define K_PO	((wide_char) (__JIS_KATAKANA_BASE + 0x3D))
#define K_MA	((wide_char) (__JIS_KATAKANA_BASE + 0x3E))
#define K_MI	((wide_char) (__JIS_KATAKANA_BASE + 0x3F))
#define K_MU	((wide_char) (__JIS_KATAKANA_BASE + 0x40))
#define K_ME	((wide_char) (__JIS_KATAKANA_BASE + 0x41))
#define K_MO	((wide_char) (__JIS_KATAKANA_BASE + 0x42))
#define K_ya	((wide_char) (__JIS_KATAKANA_BASE + 0x43))
#define K_YA	((wide_char) (__JIS_KATAKANA_BASE + 0x44))
#define K_yu	((wide_char) (__JIS_KATAKANA_BASE + 0x45))
#define K_YU	((wide_char) (__JIS_KATAKANA_BASE + 0x46))
#define K_yo	((wide_char) (__JIS_KATAKANA_BASE + 0x47))
#define K_YO	((wide_char) (__JIS_KATAKANA_BASE + 0x48))
#define K_RA	((wide_char) (__JIS_KATAKANA_BASE + 0x49))
#define K_RI	((wide_char) (__JIS_KATAKANA_BASE + 0x4A))
#define K_RU	((wide_char) (__JIS_KATAKANA_BASE + 0x4B))
#define K_RE	((wide_char) (__JIS_KATAKANA_BASE + 0x4C))
#define K_RO	((wide_char) (__JIS_KATAKANA_BASE + 0x4D))
#define K_wa	((wide_char) (__JIS_KATAKANA_BASE + 0x4E))
#define K_WA	((wide_char) (__JIS_KATAKANA_BASE + 0x4F))
#define K_WI	((wide_char) (__JIS_KATAKANA_BASE + 0x50))
#define K_WE	((wide_char) (__JIS_KATAKANA_BASE + 0x51))
#define K_WO	((wide_char) (__JIS_KATAKANA_BASE + 0x52))
#define K_N	((wide_char) (__JIS_KATAKANA_BASE + 0x53))
#define K_VU	((wide_char) (__JIS_KATAKANA_BASE + 0x54))
#define K_ka	((wide_char) (__JIS_KATAKANA_BASE + 0x55))
#define K_ke	((wide_char) (__JIS_KATAKANA_BASE + 0x56))

#ifdef __JIS_CONFIG_USE_UNICODE
#define K_LONG	((wide_char) (0x30FC))
#else
#define K_LONG	((wide_char) (0xA1BC))
#endif

/* #define J_DOT	((wide_char) (0xFF65)) /* */
#define J_WO	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x6))
#define J_a	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x7))
#define J_i	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x8))
#define J_u	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x9))
#define J_e	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0xA))
#define J_o	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0xB))
#define J_ya	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0xC))
#define J_yu	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0xD))
#define J_yo	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0xE))
#define J_tu	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0xF))
#define J_LONG	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x10))
#define J_A	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x11))
#define J_I	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x12))
#define J_U	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x13))
#define J_E	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x14))
#define J_O	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x15))
#define J_KA	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x16))
#define J_KI	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x17))
#define J_KU	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x18))
#define J_KE	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x19))
#define J_KO	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x1A))
#define J_SA	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x1B))
#define J_SI	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x1C))
#define J_SU	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x1D))
#define J_SE	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x1E))
#define J_SO	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x1F))
#define J_TA	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x20))
#define J_TI	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x21))
#define J_TU	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x22))
#define J_TE	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x23))
#define J_TO	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x24))
#define J_NA	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x25))
#define J_NI	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x26))
#define J_NU	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x27))
#define J_NE	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x28))
#define J_NO	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x29))
#define J_HA	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x2A))
#define J_HI	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x2B))
#define J_HU	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x2C))
#define J_HE	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x2D))
#define J_HO	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x2E))
#define J_MA	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x2F))
#define J_MI	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x30))
#define J_MU	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x31))
#define J_ME	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x32))
#define J_MO	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x33))
#define J_YA	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x34))
#define J_YU	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x35))
#define J_YO	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x36))
#define J_RA	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x37))
#define J_RI	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x38))
#define J_RU	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x39))
#define J_RE	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x3A))
#define J_RO	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x3B))
#define J_WA	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x3C))
#define J_N	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x3D))
#define J_NIGORI ((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x3E))
#define J_MARU	((wide_char) (__JIS_HALF_KATAKANA_BASE + 0x3F))

wide_int wide_to_hiragana(wide_int c);
wide_int wide_to_katakana(wide_int c);
/* wide_int to_wide_halfwidth_katakana(wide_int c); /**/

#define __JIS_INLINE_DEFINITION
#include "k_inline.c"

#define __JIS_INLINE_DEFINITION
#include "w_is.c"

#endif /* __JIS_KANA_H__ */
