/*
 * kd_mkind.c
 *
 * By Ross Ridge
 * Public Domain
 *
 */

#include "defs.h"
#include "kanjidic.h"
#include "linebuf.h"

#ifdef CONFIG_SCCS_IDS
static char const sccs_id[] = "@(#) myjis kd_mkind.c 1.2 98/06/01 22:28:19";
#endif

int
kanjidict::make_index() {
	quick_table = NULL;
	if (quick_table.add(quick_entry(0, 0, 0))) {
		return 1;
	}

	int lineno = 1;
	_linebuf buf;

	stroke_index = NULL;

	rewind(f);

	if (buf.read_line(f) || strncmp(buf, "# KANJ", 6) != 0) {
		printf("@@@: buf = '%s'\n", (char *) buf);
		errno = ENOEXEC;
		return lineno;
	}

	if (quick_table.add(quick_entry(0, 0, 0))) {
		return 1;
	}

	while(1) {
		if (lineno % 100 == 0) {
			putchar('.'); fflush(stdout); /* @@@ */
		}

		long offset = ftell(f);

		lineno++;

		if (buf.read_line(f)) {
			if (feof(f)) {
				break;
			}
			return lineno;
		}

		struct kanjidict::entry entry;
		if (entry.convert_line(buf, lineno)) {
			printf("@@@: convert_line buf = '%s'\n", (char *) buf);
			return lineno;
		}
		short score = entry.frequency;
		if (score == 0) {
			score = 30000;
			if (entry.grade != 0) {
				score -= (10 - entry.grade) * 100;
			}
			if (entry.halpern != 0) {
				score--;
			}
			if (entry.nelson != 0) {
				score--;
			}
			if (entry.spahn_hadamitzky != NULL) {
				score--;
			}
			if (entry.morohashi.index != 0) {
				score--;
			}
			if (entry.oindex != 0) {
				score--;
			}
		}

		if (quick_table.add(quick_entry(offset, entry.character,
						score))) {
			return 1;
		}

		if (entry.strokes < 1 || entry.strokes > 99
		    || entry.radical < 1 || entry.radical > 214) {
			errno = EINVAL;
			return 1;
		}

		if (stroke_index.extend(entry.strokes)
		    || stroke_index.select(entry.strokes - 1).add(lineno)
		    || radical_index[entry.radical - 1].add(lineno)) {
			return 1;
		}

		if (entry.skip.packed() != 0
		    && skip_index.add(entry.skip, lineno)) {
			return 1;
		}

		int first_reading = reading_index.length();
		int i;
		for (i = 0; i < entry.readings.length(); i++) {
			wide_char *start = entry.readings[i];
			if (start[0] == '-') {
				start++;
			}
			wide_char *end = wide_chr(start,
						  jis_byte_to_wide('.'));
			if (end == NULL) {
				end = start + wide_len(start);
				if (end > start && end[-1] == '-') {
					end--;
				}
			}
			size_t len = end - start;
			wide_char *dup = (wide_char *)
				malloc((len + 1) * sizeof(wide_char));

			if (dup == NULL) {
				return 1;
			}
			memcpy(dup, start, len * sizeof(wide_char));
			dup[len] = '\0';

			int j = first_reading;
			for(; j < reading_index.length(); j++) {
				if (wide_cmp(dup, reading_index[j].key) == 0) {
					free(dup);
					dup = NULL;
					break;
				}
			}
			if (dup != NULL) {
				if (reading_index.add(dup, lineno)) {
					return 1;
				}
			}
		}
		for (i = 0; i < entry.name_readings.length(); i++) {
			if (name_reading_index.add(entry.name_readings[i], 
						   lineno)) {
				return 1;
			}
			entry.name_readings.select(i) = NULL;
		}
	}
	skip_index.sort();

	qcompare<reading>::both_on_kun = 0;
	reading_index.sort();
	name_reading_index.sort();
	return 0;
}


