/*
 * kd_wrind.c
 *
 * By Ross Ridge
 * Public Domain
 *
 */

#include "defs.h"
#include "kanjidic.h"

#ifdef CONFIG_SCCS_IDS
static char const sccs_id[] = "@(#) myjis kd_wrind.c 1.2 98/06/01 22:28:21";
#endif

int
write_32(FILE *f, long val) {
	assert(val >= 0 && val <= 2147483647L);
	if (putc((unsigned char) val, f) == EOF
	    || putc((unsigned char) (val >> 8), f) == EOF
	    || putc((unsigned char) (val >> 16), f) == EOF
	    || putc((unsigned char) (val >> 24), f) == EOF) {
		return -1;
	}
	return 0;
}

int
write_16(FILE *f, long val) {
	assert(val >= 0 && val <= 65535);
	if (putc((unsigned char) val, f) == EOF
	    || putc((unsigned char) (val >> 8), f) == EOF) {
		return -1;
	}
	return 0;
}

inline int
write_wchar(FILE *f, wide_char wc) {
#ifdef __JIS_CONFIG_USE_UNICODE
	return write_32(f, wc);
#else
	return write_16(f, wc);
#endif
}

template <class TYPE>
int
write_ent(FILE *f, list<TYPE> const &lst) {
	int len = lst.length();
	if (write_16(f, len) == -1) {
		return -1;
	}
	int i;
	for (i = 0; i < len; i++) {
		if (write_ent(f, lst[i]) == -1) {
			return -1;
		}
	}
	return 0;
}

template <class TYPE>
int
write_ent(FILE *f, element<TYPE> const &lst) {
	if (write_ent(f, lst.key) == -1
	    || write_ent(f, lst.line) == -1) {
		return -1;
	}
	return 0;
}

template <class TYPE>
int
write_ent(FILE *f, index<TYPE> const &lst) {
	return write_ent(f, (list<element<TYPE>/**/> const &) lst);
}

inline int
write_ent(FILE *f, int n) {
	return write_16(f, n);
}

inline int
write_ent(FILE *f, skip_code sc) {
	return write_16(f, sc.packed());
}

inline int
write_ent(FILE *f, wide_char const *ws) {
	size_t len = wide_len(ws);
	if (write_16(f, len) == -1) {
		return -1;
	}
	int i;
	for (i = 0; i < (int) len; i++) {
		if (write_wchar(f, ws[i]) == -1) {
			return -1;
		}
	} 
	return 0;
}

inline int
write_ent(FILE *f, struct kanjidict::quick_entry const &qent) {
	if (write_32(f, qent.offset) == -1
	    || write_wchar(f, qent.character) == -1
	    || write_16(f, qent.score) == -1) {
		return -1;
	}
	return 0;
}

int
write_radical_index(FILE *f, list<int> const *lsts) {
	int i;
	for (i = 0; i < 214; i++) {
		if (write_ent(f, lsts[i]) == -1) {
			return -1;
		}
	}
	return 0;
}

int
kanjidict::write_index(char const *ifile) const {
	FILE *f = fopen(ifile, "wb");
	if (f == NULL) {
		return -1;
	}

	if (write_ent(f, quick_table) == -1
	    || write_ent(f, skip_index) == -1
	    || write_ent(f, stroke_index) == -1
	    || write_radical_index(f, radical_index) == -1
	    || write_ent(f, (index<reading> &) reading_index) == -1
	    || write_ent(f, (index<reading> &) name_reading_index)
	       == -1) {
		return -1;
	}

	if (fclose(f) == EOF) {
		return -1;
	}

	return 0;
}
					  
/* Local variables: */
/* mode: c++ */
/* End: */


