/*
 * list.c
 *
 * By Ross Ridge 
 * Public Domain
 *
 * Member functions of the template list.
 *
 * @(#) myjis list.c 1.1 97/02/21 01:44:13
 *
 */

#ifndef __LIST_C__
#define __LIST_C__

template <class TYPE>
void
list<TYPE>::destroy() {
	if (len != 0) {
		delete[] array;
	}
}

template <class TYPE>
int
list<TYPE>::copy(list<TYPE> const &l) {
  	len = l.len;
	left = 0;
	if (len > 0) {
		array = new TYPE[len];
		if (array == NULL) {
			len = 0;
			return 1;
		}
		int i;
		for(i = 0; i < len; i++) {
			array[i] = l.array[i];
		}
	}
	return 0;
}

template <class TYPE>
list<TYPE> &
list<TYPE>::operator =(list<TYPE> const &l) {
	if (this != &l) {
		destroy();
		if (copy(l)) {
			abort();
		}
	}
	return *this;
}
	
template <class TYPE>
int
list<TYPE>::grow(int new_left) {
#if 0
	printf("@@@: %d %p new_left %d, left %d, len %d\n",
	       sizeof(TYPE), (void *) this,
	       new_left, left, len);
#endif

	TYPE *new_array = new TYPE[len + new_left];
	if (new_array == NULL) {
		return 1;
	}
	left = new_left;
	if (len != 0) {
		int i;
		for(i = 0; i < len; i++) {
			new_array[i] = array[i];
		}
		delete[] array;
	}
	array = new_array;
	return 0;
}

template <class TYPE>
int
list<TYPE>::extend(int size) {
#if 0
	printf("@@@: %d, %p, size %d, left %d, len %d\n",
	       sizeof(TYPE), (void *) this,
	       size, left, len);
#endif
	int needed = size - len;
	if (needed <= 0) {		   /* Too small */
		return 0;
	}
	if (needed <= left) {		   /* Fits in existing allocation */
		left -= needed;
		len = size;
		return 0;
	}
	int new_left = left + len + left;  /* Double allocation */
	if (needed > new_left) {
		new_left = needed;
	}
	if (grow(new_left)) {
		return 1;
	}
	left -= needed;
	len = size;
	return 0;
}

template <class TYPE>
int
list<TYPE>::add(TYPE const &ent) {
	if (left == 0) {
		int new_left = len;
		if (new_left == 0) {
			new_left = 4;
		} 
		if (grow(new_left)) {
			return 1;
		}
	}
	array[len++] = ent;
	left--;
	return 0;
}

template <class TYPE>
auto_free_list<TYPE>::~auto_free_list() {
	int i;
	for (i = 0; i < len; i++) {
		TYPE p = array[i];
		if (p != NULL) {
			free(p);
		}
	}
}

#endif /* __LIST_C__ */

/* Local variables: */
/* mode: c++ */
/* End: */
