/*
 * myjconv.c
 *
 * By Ross Ridge
 * Public Domain
 *
 * Japanese character set conversion utility.
 *
 */

#include "defs.h"

#include "version.c"

static char const myjconv_sccs_id[] = "@(#) myjis myjconv.c 1.5 98/03/18 21:58:39";

static int
usage(char const *prg) {
	fprintf(stderr,
		"usage: %s [-lV] -f from-code-set -t to-code-set [filename]\n",
		prg);
	return 1;
}

int
main(int argc, char **argv) {
	int c;
	char const *from = NULL;
	char const *to = NULL;
	FILE *f;
	int i;
	char const *ifile;
	char const *itmp;
	char *otmp;
	size_t ilen, olen;
	size_t n;
	char ibuf[512], obuf[512];
	jis_conv_state conv;
	int got_eof = 0;
	int ilseq = 0;
	int save_errno;
		
	setlocale(LC_ALL, "");

	do {
		c = getopt(argc, argv, "Vf:t:l");
		switch(c) {
		case 'V':
			version(myjconv_sccs_id);
			return 0;
			
		case 'f':
			from = optarg;
			break;

		case 't':
			to = optarg;
			break;

		case 'l':
			i = 0;
			while(__jis_code_name_table[i].name != NULL) {
				puts(__jis_code_name_table[i].name);
				i++;
			}
			return 0;

		case '?':
			return usage(argv[0]);
		}
	} while(c != EOF);

	if (optind == argc) {
		f = stdin;
		ifile = "stdin";
	} else if (optind == argc - 1) {
		ifile = argv[optind];
		f = fopen(ifile, "rb");
		if (f == NULL) {
			perror(ifile);
			return 1;
		}
	} else {
		return usage(argv[0]);
	}

	if (from == NULL || to == NULL) {
		return usage(argv[0]);
	}

	conv = jis_conv_open(to, from);
	if (conv == NULL) {
		if (errno == EINVAL) {
			fprintf(stderr, "%s: code set not supported.\n",
				argv[0]);
			return 1;
		}
		perror("jis_conv_open");
		return 1;
	}
		    
	ilen = 0;

	while(1) {
		if (got_eof) {
			n = 0;
		} else {
			n = fread(ibuf + ilen, 1, sizeof ibuf - ilen, f);
		}
		if (n == 0) {
			if (ferror(f)) {
				perror(ifile);
				return 1;
			}
			if (ilen == 0) {
				return 0;
			}
		}
		itmp = ibuf;
		ilen += n;
		otmp = obuf;
		olen = sizeof obuf;
#if 0
		fprintf(stderr, "@@@: n = %ld, itmp = %p, ilen %ld"
				" otmp = %p, olen = %ld\n",
			(long) n, (void *) itmp, (long) ilen,
			(void *) otmp, (long) olen);
#endif
		n = jis_conv(conv, &itmp, &ilen, &otmp, &olen);
		save_errno = errno;
#if 0
		fprintf(stderr, "@@@: n = %ld, itmp = %p, ilen %ld"
				" otmp = %p, olen = %ld\n",
			(long) n, (void *) itmp, (long) ilen,
			(void *) otmp, (long) olen);
		if (n == -1) {
			errno = save_errno;
			perror("@@@");
		}
#endif
		if (otmp > obuf && fwrite(obuf, (size_t) (otmp - obuf), 1,
					  stdout) != 1) {
			perror("stdout");
			return 1;
		}
		if (n == (size_t) -1) {
			assert(ilen != 0);

			switch(save_errno) {
			case EILSEQ:
				if (!ilseq) {
#if 0
					putc('"', stderr);
					fwrite(ibuf, itmp - ibuf, 1, stderr);
					fputs("\"\n\"", stderr);
					fwrite(itmp, ilen, 1, stderr);
					putc('"', stderr);
#endif
					fprintf(stderr,
						"%s: illegal multibyte"
						" sequence in input.\n",
						ifile);
					ilseq = 1;
				}
				ilen--;
				memmove(ibuf, itmp + 1, ilen);
				break;

			case EINVAL:
				if (got_eof) {
					fprintf(stderr,
						"%s: incomplete multibyte"
						" sequence at end of input.\n",
						ifile);
					return 1;
				}
				memmove(ibuf, itmp, ilen);
				break;

			case E2BIG:
				if (olen == sizeof obuf) {
					fprintf(stderr,
						"internal buffer too small"
						" for conversion!!!\n");
					return 1;
				}
				memmove(ibuf, itmp, ilen);
				break;

			default:
				perror("jis_conv");
				return 1;
			}
		}
	}
}








