/*
 * pts_log.c
 *
 * By Ross Ridge
 * Public Domain
 *
 */

#include "defs.h"
#include "pts.h"

#include <stdarg.h>

static char const sccs_id[] = "@(#) myjis pts_log.c 1.1 99/08/25 18:32:05";

FILE *flog = NULL;

void
log(char const *fmt, ...) {
	if (flog == NULL) {
		return;
	}

	int save_errno = errno;

	va_list ap;
	va_start(ap, fmt);
	vfprintf(flog, fmt, ap);
	va_end(ap);

	errno = save_errno;
}

void
log_buf(char const *buf, size_t len) {
	size_t i;
	for (i = 0; i < len; i++) {
		char c = buf[i];
		switch(c) {
		case '\a':
			log("\\a");
			break;
		case '\b':
			log("\\b");
			break;
		case '\n':
			log("\\n");
			break;
		case '\r':
			log("\\r");
			break;
		case '\t':
			log("\\t");
			break;
		case '\0':
			log("\\0");
			break;
		case '\\':
			log("\\\\");
			break;
		default:
			if ((c >= 0 && c < 32)
			    || ((unsigned char) c >= 127
				&& (unsigned char) c <= 255)) {
				log("\\%03o", (unsigned char) c);
			} else {
				log("%c", c);
			}
			break;
		}
	}
}

void
log_list_int(list<int> const &lst) {
	int i;
	for (i = 0; i < lst.length(); i++) {
		if (i != 0) {
			log(" ");
		}
		log("%d", lst[i]);
	}
}

void
lerror(char const *msg) {
	int save_errno = errno;

	perror(msg);

	if (flog != NULL) {
		char const *str = strerror(save_errno);
		fprintf(flog, "%s: %s\n", msg, str);
	}

	errno = save_errno;
}
