/*
 * reduce.y
 *
 * By Ross Ridge
 * Public Domain
 *
 * Try to reduce a Japanese word to dictionary form by removing and
 * replacing suffixes.
 *
 */

%union
{
	struct list {
		char **list;
		int pos;
		int len;
		int err;
	} *value;
}

%{
#include "defs.h"
#include "edict.h"

#include <stdarg.h>

static char const sccs_id[] = "@(#) myjis reduce.y 1.12 99/01/15 19:18:41";

void yyerror(char const *s);
int yylex(void);

struct list *replace_suffix(struct list *s, wide_int wc);
struct list *print_word(struct list *s, wide_int wc, ...);

int not_start_flag;
wide_char *input_string;
wide_char *suffix_point;
wide_char *lex_pos;

extern int yydebug;

static struct list list_err = { NULL, 0, 0, ENOMEM };
struct list *the_value;

/* #define YYDEBUG 1 /* */
%}

%token T_a
%token T_A
%token T_i
%token T_I
%token T_u
%token T_U
%token T_e
%token T_E
%token T_o
%token T_O
%token T_KA
%token T_GA
%token T_KI
%token T_GI
%token T_KU
%token T_GU
%token T_KE
%token T_GE
%token T_KO
%token T_GO
%token T_SA
%token T_ZA
%token T_SI
%token T_ZI
%token T_SU
%token T_ZU
%token T_SE
%token T_ZE
%token T_SO
%token T_ZO
%token T_TA
%token T_DA
%token T_TI
%token T_DI
%token T_tu
%token T_TU
%token T_DU
%token T_TE
%token T_DE
%token T_TO
%token T_DO
%token T_NA
%token T_NI
%token T_NU
%token T_NE
%token T_NO
%token T_HA
%token T_BA
%token T_PA
%token T_HI
%token T_BI
%token T_PI
%token T_HU
%token T_BU
%token T_PU
%token T_HE
%token T_BE
%token T_PE
%token T_HO
%token T_BO
%token T_PO
%token T_MA
%token T_MI
%token T_MU
%token T_ME
%token T_MO
%token T_ya
%token T_YA
%token T_yu
%token T_YU
%token T_yo
%token T_YO
%token T_RA
%token T_RI
%token T_RU
%token T_RE
%token T_RO
%token T_wa
%token T_WA
%token T_WI
%token T_WE
%token T_WO
%token T_N
%token T_VU
%token T_ka
%token T_ke

%token START

%type <value> suffix ambiguous_endings verb_ending consonant_endings
%type <value> irregular_verbs start;

%%

start: suffix { the_value = $$; }

suffix : verb_ending 
	| adjective_endings		{ $$ = replace_suffix(NULL, K_I); }
	| ambiguous_endings
	| START irregular_verbs		{ $$ = $2; };

/*
 * All the reduce/reduce conflicts in this grammar are superceded
 * by this rule (which introduces a new reduce/reduce conflict for
 * each case).  We handle the ambiguity these cases represent in
 * the Japanese language by returning both possible suffixes.
 */
 
ambiguous_endings : T_RE T_BA 		{ $$ = replace_suffix(NULL, K_RU); }
	| T_KE T_RE T_BA		{ $$ = replace_suffix(NULL, K_I);
					  $$ = replace_suffix($$, K_KU); }
	| T_RA T_RE vowel_stem_cont	{ $$ = replace_suffix(NULL, K_RU); }
	| T_ME vowel_stem_cont		{ $$ = replace_suffix(NULL, K_MU);
					  $$ = replace_suffix($$, K_I); }
	| T_TE				{ $$ = replace_suffix(NULL, K_TU);
					  $$ = replace_suffix($$, K_RU); }

verb_ending : consonant_endings
	| vowel_stem_endings {
		wide_char wc;

		if (suffix_point > input_string) {
			wc = wide_to_katakana_vowel(suffix_point[-1]);
			if (wc == K_E || wc == K_I) {
				$$ = replace_suffix(NULL, K_RU);
			}
		}
	};

vowel_stem_endings: continuative_endings | negative_endings | te_endings
	| T_RE T_BA | T_YO T_U | T_RO
	| T_RA T_RE vowel_stem_cont | T_SA T_SE vowel_stem_cont

vowel_stem_cont: T_RU | vowel_stem_endings;

continuative_endings: masu_endings | T_TA adjective_cont
	| dasu_endings | deru_endings | hajimeru_endings | mawaru_endings;

masu_endings: T_MA T_SE T_N | T_MA T_SU T_RE T_BA | T_MA T_SI T_yo T_U
	| T_MA T_SI te_endings | T_MA T_SU;

dasu_endings: T_DA T_SU
	 | T_DA su_endings; 

deru_endings: T_DE vowel_stem_cont;

hajimeru_endings: T_HA T_ME vowel_stem_cont;

mawaru_endings: T_MA T_WA T_RU | T_MA T_WA ru_endings | T_MA T_WA tte_endings;

negative_endings: T_NA adjective_cont;

te_endings: T_TE | T_TA T_RA | T_TA T_RI | T_TA | chau_endings;

chau_endings: T_TI T_ya T_U | T_TI T_ya wu_endings | T_TI T_ya tte_endings;

consonant_endings:
	ku_endings		 	{ $$ = replace_suffix(NULL, K_KU); }
	| su_endings 			{ $$ = replace_suffix(NULL, K_SU); }
	| tu_endings			{ $$ = replace_suffix(NULL, K_TU); }
	| nu_endings			{ $$ = replace_suffix(NULL, K_NU); }
	| mu_endings			{ $$ = replace_suffix(NULL, K_MU); }
	| ru_endings			{ $$ = replace_suffix(NULL, K_RU); }
	| bu_endings			{ $$ = replace_suffix(NULL, K_BU); }
	| gu_endings			{ $$ = replace_suffix(NULL, K_GU); }
	| wu_endings			{ $$ = replace_suffix(NULL, K_U); }
	| nde_endings			{ $$ = replace_suffix(NULL, K_NU);
					  $$ = replace_suffix($$, K_MU);
					  $$ = replace_suffix($$, K_BU); }
	| tte_endings			{ $$ = replace_suffix(NULL, K_RU);
					  $$ = replace_suffix($$, K_TU);
					  $$ = replace_suffix($$, K_U);};

nde_endings: T_N de_endings;

tte_endings: T_tu te_endings;

de_endings: T_DE | T_DA T_RA | T_DA T_RI | T_DA | jau_endings;

jau_endings: T_ZI T_ya T_U | T_ZI T_ya wu_endings | T_ZI T_ya tte_endings;

ku_endings: T_KI continuative_endings | T_KA negative_endings
	| T_KE T_BA | T_KO T_U | T_KE vowel_stem_cont;
	| T_KA consonant_other | T_I te_endings | T_KE;

su_endings: T_SI continuative_endings | T_SA negative_endings
	| T_SE T_BA | T_SO T_U | T_SE vowel_stem_cont;
	| T_SA consonant_other | T_SI te_endings | T_SE;

tu_endings: T_TI continuative_endings | T_TA negative_endings
	| T_TE T_BA | T_TO T_U | T_TE vowel_stem_cont;
	| T_TA consonant_other | T_TE;

nu_endings: T_NI continuative_endings | T_NA negative_endings
	| T_NE T_BA | T_NO T_U | T_NE vowel_stem_cont;
	| T_NA consonant_other;

mu_endings: T_MI continuative_endings | T_MA negative_endings
	| T_ME T_BA | T_MO T_U | T_ME vowel_stem_cont;
	| T_MA consonant_other | T_ME;

ru_endings: T_RI continuative_endings | T_RA negative_endings
	| T_RE T_BA | T_RO T_U | T_RE vowel_stem_cont;
	| T_RA consonant_other | T_RE;

bu_endings: T_BI continuative_endings | T_BA negative_endings
	| T_BE T_BA | T_BO T_U | T_BE vowel_stem_cont;
	| T_BA consonant_other | T_BE;

gu_endings: T_GI continuative_endings | T_GA negative_endings
	| T_GE T_BA | T_GO T_U | T_GE vowel_stem_cont;
	| T_GA consonant_other | T_I de_endings | T_GE;

wu_endings: T_I continuative_endings | T_WA negative_endings
	| T_E T_BA | T_O T_U | T_E vowel_stem_cont;
	| T_WA consonant_other | T_E;

consonant_other: T_RE vowel_stem_cont | T_SA vowel_stem_cont;

adjective_cont: T_I | adjective_endings;
	
adjective_endings: T_KU | T_KU T_NA T_I | T_KU T_TE | T_KA T_tu T_TA
	| T_KE T_RE T_BA | meru_endings;

meru_endings: T_ME vowel_stem_cont;

irregular_verbs:
	suru			{ $$ = print_word(NULL, 
						  K_SU, K_RU, (wide_char) 0); }
	| kuru			{ $$ = print_word(NULL,
						  K_KU, K_RU, (wide_char) 0); }
/*
	| desu			{ $$ = print_word(NULL,
						  K_DE, K_SU, (wide_char) 0); }
	| da			{ $$ = print_word(NULL,
						  K_DA, (wide_char) 0); }
*/
	;

suru: T_SI continuative_endings | T_SI negative_endings 
	| T_SI te_endings | T_SU T_RE T_BA | T_SI T_YO T_U | T_SI T_RO
	| T_SA T_RE vowel_stem_cont | T_SA T_SE vowel_stem_cont;

kuru: T_KI continuative_endings | T_KO negative_endings | T_KI te_endings
	| T_TU T_RE T_BA | T_KO T_YO T_U | T_KO T_I 
	| T_KO T_RA T_RE vowel_stem_cont | T_KO T_SA T_SE vowel_stem_cont;

/*
desu: T_DE T_WA | T_ZI T_ya | T_DE T_SI T_TA T_RA T_BA
	| T_DE T_SI te_endings | T_DE T_SI T_yo T_U;

da: T_DE negative_endings | T_ZI T_ya negative_endings | T_DE
	| T_DA T_tu T_TA | T_NA T_RA | T_NA T_RA T_BA | T_DE T_RO T_U;
*/

%%

void
yyerror(char const *msg) {
#if YYDEBUG
	printf("%s\n", msg);
#endif
}

int
yylex(void) {
	wide_char wc;

	if (!not_start_flag) {
		not_start_flag = 1;
		return START;
	}

	wc = wide_to_katakana(*lex_pos++);
		
	if (wide_is_katakana(wc)) {
		return (wc - K_a) + T_a;
	}
	if (wc == 0) {
		return wc;
	}
	return '?';
}

static void
list_free(struct list *list) {
	int i;

	if (list == NULL || list == &list_err) {
		return;
	}
	if (list->list != NULL) {
		for(i = 0; i < list->pos; i++) {
			free(list->list[i]);
		}
	}
	free(list->list);
	free(list);
}

static struct list *
list_error(struct list *list, int err) {
	if (list != NULL) {
		list_free(list);
	}
	if (err == 0) {
		err = EINVAL;
	} 
	list_err.err = err;
	return &list_err;
}

static struct list *
add_word_to_list_2(struct list *list, char *s) {
	if (list == NULL) {
		list = malloc(sizeof(struct list));
		if (list == NULL) {
			list_err.err = ENOMEM;
			return &list_err;
		}
		list->len = 8;
		list->pos = 0;
		list->err = 0;
		list->list = malloc(list->len * sizeof(char *));
		if (list->list == NULL) {
			return list_error(list, ENOMEM);
		}
	}

	if (list->err != 0) {
		return list;
	}

	if (list->pos == list->len) {
		char **p;
		int newlen = list->len * 2;
		p = realloc(list->list, newlen * sizeof(char *));
		if (p == NULL) {
			return list_error(list, ENOMEM);
		}
		list->list = p;
		list->len = newlen;
	}

#if 0
	printf("@@@ adding %s\n", s == NULL ? "NULL" : s);
#endif

	list->list[list->pos++] = s;

	return list;
}

static struct list *
add_word_to_list(struct list *list, wide_char *ws) {
	char *s = wide_to_jis_dup(ws);
	if (s == NULL) {
		return list_error(list, errno);
	}
	return add_word_to_list_2(list, s);
}

static struct list *
add_list_to_list(struct list *dest, struct list *src) {
	int i;

	if (dest == &list_err) {
		list_free(src);
		return dest;
	}
	if (src == &list_err) {
		list_free(dest);
		return src;
	}
	for (i = 0; i < src->pos; i++) {
		dest = add_word_to_list_2(dest, src->list[i]);
	}
	free(src->list);
	free(src);

	return dest;
}

struct list *
replace_suffix(struct list *list, wide_int wc) {
	size_t len = suffix_point - input_string;
	wide_char *ws;

	if (len == 0) {
		return list;
	}

	ws = malloc((len + 2) * sizeof(wide_char));
	if (ws == NULL) {
		return list_error(list, ENOMEM);
	}

	memcpy(ws, input_string, len * sizeof(wide_char));
	if (len > 0 && wide_is_hiragana(ws[len - 1])) {
		wc = wide_to_hiragana(wc);
	}		
	ws[len] = wc;
	ws[len + 1] = (wide_char) 0;

	list = add_word_to_list(list, ws);
	free(ws);
	return list;
}

struct list *
print_word(struct list *list, wide_int wc, ...) {
	wide_char buf[10];
	int i;
	va_list ap;

	va_start(ap, wc);
	i = 0;
	while(wc != (wide_char) 0) {
		buf[i++] = wc;
		if (i == 9) {
			return list_error(list, EFAULT);
		}
		wc = va_arg(ap, wide_char);
	}
	va_end(ap);
	buf[i] = wc;
	if (wide_is_hiragana(input_string[0])) {
		wide_to_hiragana_str(buf);
	}
	return add_word_to_list(list, buf);
}
	
char **
edict_reduce(char const *word) {
	wide_char *ws;
	size_t len;
	size_t j;
	char **ret;
	struct list *list = NULL;
	

#if YYDEBUG 
	yydebug = 1;
#endif

	ws = jis_to_wide_dup(word);
	if (ws == NULL) {
		return NULL;
	}
	wide_to_hiragana_vowel_str(ws);
	len = wide_len(ws);
	input_string = ws;
	for(j = 0; j < len; j++) {
		/*
		 * Initialize lexer variables.
		 */
		not_start_flag = (j != 0);
		suffix_point = lex_pos = ws + j;
		
		the_value = NULL;
		if (!yyparse() && the_value != NULL) {
			if (the_value == &list_err) {
				list_free(list);
				errno = list_err.err;
				return NULL;
			}
			list = add_list_to_list(list, the_value);
			if (list == &list_err) {
				errno = list_err.err;
				return NULL;
			}
		}
			
		if (wide_to_katakana(ws[j]) == K_tu) {
			j++;
		}
	}

	list = add_word_to_list_2(list, NULL);
	if (list == &list_err) {
		errno = list_err.err;
		return NULL;
	}

#if 0
	printf("@@@ list->pos = %d\n", list->pos);
#endif

	ret = list->list;
	free(list);
	return ret;
}
