/*
 * w_is.c
 *
 * By Ross Ridge
 * Public Domain
 *
 * Implementation of wide_is_hiragana() and wide_is_katakana().
 *
 */

#include "inline.h"

__JIS_INLINE_SCCS_ID("@(#) myjis w_is.c 1.3 98/06/01 22:28:30")

#ifdef __JIS_CONFIG_USE_UNICODE

__JIS_INLINE_FUNCTION(
int,
wide_is_hiragana, (wide_int wc), {
	return wc >= 0x3041 && wc <= 0x3093;
})

__JIS_INLINE_FUNCTION(
int,
wide_is_katakana, (wide_int wc), {
	return wc >= 0x30A1 && wc <= 0x30F6;
})

__JIS_INLINE_FUNCTION(
int,
wide_is_han, (wide_int wc), {
	return wc >= 0x4E00 && wc <= 0x9FFF;
})

#if 0
__JIS_INLINE_FUNCTION(
int,
wide_is_halfwidth_katakana, (wide_int wc), {
	return wc >= 0xFF66 && wc <= 0xFF9F;
})
#endif

#define wide_is_hiragana(wc) __JIS_INLINE_NAME(wide_is_hiragana)(wc)
#define wide_is_katakana(wc) __JIS_INLINE_NAME(wide_is_katakana)(wc)
#define wide_is_han(wc) __JIS_INLINE_NAME(wide_is_han)(wc)

#else /* __JIS_CONFIG_USE_UNICODE */

__JIS_INLINE_FUNCTION(
int,
wide_is_hiragana, (wide_int wc), {
	return wc >= (wide_char) 0xA4A1 && wc <= (wide_char) 0xA4F3;
})

__JIS_INLINE_FUNCTION(
int,
wide_is_katakana, (wide_int wc), {
	return wc >= (wide_char) 0xA5A1 && wc <= (wide_char) 0xA5F6;
})

__JIS_INLINE_FUNCTION(
int,
wide_is_han, (wide_int wc), {
	unsigned byte;

	if (wc > 0xFFFF) {
		return EOF;
	}

        byte = (unsigned) wc & 0xFF00;
	if (byte >= 0xA100 && byte <= 0xFE00) {
		byte = wc & 0x00FF;
		if ((byte >= 0x00A1 && byte <= 0x00FE)
		    || (byte >= 0x0021 && byte <= 0x007E)) {
			return 1;
		}
	}
	return 0;
})

#define wide_is_hiragana(wc) __JIS_INLINE_NAME(wide_is_hiragana)(wc)
#define wide_is_katakana(wc) __JIS_INLINE_NAME(wide_is_katakana)(wc)

#endif /* __JIS_CONFIG_USE_UNICODE */

