/*
 * w_ktor.c
 *
 * By Ross Ridge
 * Public Domain
 *
 * Implementation of wide_kana_to_roman().
 *
 */

#include "defs.h"

#ifdef CONFIG_SCCS_IDS
static char const sccs_id[] = "@(#) myjis w_ktor.c 1.2 98/03/18 21:58:44";
#endif

static int
k2r_sort_compare(void const *v1, void const *v2) {
	struct jis_r2k_kana const *p1 = v1;
	struct jis_r2k_kana const *p2 = v2;
	int n;

	n = wide_cmp(p1->kana, p2->kana);
	if (n == 0) {
		return (p1->flags & (JIS_R2K_OBSOLETE | JIS_R2K_KATAKANA 
				     | JIS_R2K_PARTICLE))
			- (p2->flags & (JIS_R2K_OBSOLETE | JIS_R2K_KATAKANA
					| JIS_R2K_PARTICLE));
	}
	return n;
}

static struct jis_r2k_kana *
make_k2r_table(void) {
	struct jis_r2k_kana *table;

	table = malloc(R2K_TABLE_LEN * sizeof(struct jis_r2k_kana));
	if (table == NULL) {
		return table;
	}

	memcpy(table, __jis_r2k_table,
	       sizeof(struct jis_r2k_kana) * R2K_TABLE_LEN);
	qsort(table, R2K_TABLE_LEN, sizeof(struct jis_r2k_kana),
	      k2r_sort_compare);

	return table;
}

static int
k2r_search_compare(void const *v1, void const *v2) {
	wide_char const *w1 = v1;
	struct jis_r2k_kana const *p2 = v2;

#if 0
	printf("@@@: search roman = %s kana = %04lX %04lX %04lX"
	       " flags = %03o\n", p2->roman, (long) p2->kana[0],
	       (long) p2->kana[1], (long) p2->kana[2], p2->flags);
#endif
	return wide_cmp(w1, p2->kana);
}
	

int
wide_kana_to_roman(struct jis_r2k_kana const **roman,
		   wide_char const *ws, int flags) {
	static struct jis_r2k_kana *k2r_table;
	struct jis_r2k_kana *table;
	wide_char key[__JIS_R2K_MAX_KANA_LEN + 1];
	int i;
	struct jis_r2k_kana *p;

	if ((flags & JIS_R2K_HEPBURN) || (flags & JIS_R2K_ALL) == 0) {
		flags = (flags & ~JIS_R2K_ALL) | JIS_R2K_HEPBURN;
	} else if (flags & JIS_R2K_KUNREI) {
		flags = (flags & ~JIS_R2K_ALL) | JIS_R2K_KUNREI;
	} else if (flags & JIS_R2K_NIPPON) {
		flags = (flags & ~JIS_R2K_ALL) | JIS_R2K_NIPPON;
	}

	table = k2r_table;
	if (table == NULL) {
		k2r_table = table = make_k2r_table();
	}

	i = 0;
	while(i < __JIS_R2K_MAX_KANA_LEN && ws[i] != (wide_char) 0) {
		key[i] = wide_to_katakana(ws[i]);
		i++;
	}

	while(i > 0) {
		key[i] = (wide_char) 0;
		p = bsearch((void const *) key, (void const *) table,
			    R2K_TABLE_LEN, sizeof(struct jis_r2k_kana),
			    k2r_search_compare);
		if (p != NULL) {
			while(p > table
			      && k2r_search_compare(key, p - 1) == 0) {
				p--;
			}
			do {
#if 0
				printf("@@@: roman = %s"
				       " kana = %04lX %04lX %04lX"
				       " flags = (%03o, %03o)\n",
				       p->roman, (long) p->kana[0],
				       (long) p->kana[1], (long) p->kana[2],
				       p->flags, flags);
#endif
				if ((p->flags & flags)
				    && (!(p->flags & JIS_R2K_PARTICLE)
					|| (flags & JIS_R2K_PARTICLE))) {
					*roman = p;
					return 1;
				}
				p++;
			} while(p < table + R2K_TABLE_LEN
				&& k2r_search_compare(key, p) == 0);
		}
		i--;
	}
	
	if (key[0] == K_N) {
		switch(wide_to_katakana(ws[1])) {
		case K_A:	
		case K_E:
		case K_I:
		case K_O:
		case K_U:
		case K_YA:
		case K_YO:
		case K_YU:
			*roman = &__jis_r2k_n_;
			break;

		default:
			*roman = &__jis_r2k_n;
			break;
		}
		return 1;
	}

	if (key[0] == K_LONG) {
		*roman = &__jis_r2k_long;
		return 1;
	}

	return 0;
}
