/*
 * w_rtokl.c
 *
 * By Ross Ridge
 * Public Domain
 *
 */

#include "defs.h"

#ifdef CONFIG_SCCS_IDS
static char const sccs_id[] = "@(#) myjis w_rtokl.c 1.1 97/01/23 09:34:46";
#endif

static int
r2ks(wide_char **list, size_t list_len, 
     wide_char const *prev, size_t prev_len, wide_char const *kana,
     wide_char const *src, int flags, int *err) {
	wide_char *dest;
	size_t dpos;
	size_t dest_len;
	int ret;
	size_t kana_len;

	if (list_len == 0) {
		*err |= JIS_R2K_ERR_AMBIGUOUS;
		return 0;
	}

	kana_len = wide_len(kana);
	dest_len = prev_len + kana_len + wide_len(src);
	dest = malloc((dest_len + 1) * sizeof(wide_char));
	if (dest == NULL) {
		return -1;
	}
	list[0] = dest;

	memcpy(dest, prev, prev_len * sizeof(wide_char));
	memcpy(dest + prev_len, kana, kana_len * sizeof(wide_char));

	dpos = prev_len + kana_len;
	ret = 1;

	while(*src != '\0') {
		struct jis_r2k_kana const *kana_list[8];
		int i;
		wide_char const *kana;
		size_t kana_len;
		int matches;

		matches = wide_roman_to_kana(kana_list, 8, src, flags);

		for (i = 1; i < matches; i++) {
			int r;
			r = r2ks(list + ret, list_len - ret,
				 dest, dpos, kana_list[i]->kana, 
				 src + strlen(kana_list[i]->roman),
				 flags, err);
			if (r == 0) {
				break;
			}
			ret += r;
		}

		if (matches <= 0) {
			kana = src;
			kana_len = 1;
			src++;
			*err |= JIS_R2K_ERR_UNKNOWN;
		} else {
			kana = kana_list[0]->kana;
			kana_len = wide_len(kana);
			src += strlen(kana_list[0]->roman);
		}

		while(dpos + kana_len > dest_len) {
			wide_char *d;

			dest_len *= 2;
			d = realloc(dest, dest_len * sizeof(wide_char));
			if (d == NULL) {
				int j;
				for(j = 0; j < ret; j++) {
					free(list[j]);
				}
				return -1;
			}

			list[0] = dest = d;
		}

		memcpy(dest + dpos, kana, kana_len * sizeof(wide_char));
		dpos += kana_len;
	}

	dest[dpos] = (wide_char) 0;

	return ret;
}

int
wide_roman_to_kana_list(wide_char **list, size_t list_len, 
			wide_char const *src, int flags, int *err) {
	const wide_char nul = 0;
	*err = 0;
	return r2ks(list, list_len, &nul, 0, &nul, src, flags, err);
}
