/*
 * w_tobyte.c
 *
 * By Ross Ridge
 * Public Domain
 *
 * Implementation of jis_wide_to_byte().
 *
 */

#include "defs.h"

#ifdef CONFIG_SCCS_IDS
static char const sccs_id[] = "@(#) myjis w_tobyte.c 1.10 98/06/01 22:28:32";
#endif

#ifdef __JIS_CONFIG_USE_UNICODE

static int
wide_to_roman(wide_int wc) {
	int n;

	if (wc < 0 || wc > 0xFFFF) {
		return EOF;
	}

	n = __jis_wide_to_jis[wc / 256][wc % 256];

	if (n == -1) {
		if (wc < 0 || (wc >= 32 && wc <= 126) || wc >= 160) {
			return EOF;
		}
		return (int) wc;
	}
	return n;
}

#else /* __JIS_CONFIG_USE_UNICODE */

static int
wide_to_roman(wide_int wc) {
	if ((unsigned long) wc >= 256) {
		return -1;
	}
	return (int) wc;
}

#endif /* __JIS_CONFIG_USE_UNICODE */

int
jis_wide_to_byte(wide_int wc) {
	int n;

	if (wc == WIDE_EOF) {
		return EOF;
	}

	switch(__jis_locale_ctype) {
	case JIS_LOCALE_ASCII:
		if ((unsigned long) wc <= 255) {
			return (char) wc;
		}
		return EOF;

	case JIS_LOCALE_JIS_ROMAN:
		return wide_to_roman(wc);

	case JIS_LOCALE_JIS:
		n = wide_to_roman(wc);
		if (n >= 161 && n <= 223) {
			return EOF;
		}
		return n;

	case JIS_LOCALE_EUC:
		if (((unsigned long) wc <= 161 && wc != 142 && wc != 143)
		    || wc == 255) {
			return (char) wc;
		}
		return EOF;

	case JIS_LOCALE_SHIFT_JIS:
		n = wide_to_roman(wc);
		if ((n >= 0 && n <= 161) || (n >= 160 && n <= 223)
		    || (n >= 240 && n <= 255)) {
			return (char) n;
		}
		return EOF;

#ifdef __JIS_CONFIG_HAVE_UNICODE
	case JIS_LOCALE_UTF8:
		if (wc >= 0 && wc <= 127) {
			return (char) wc;
		}
		return EOF;
#endif

	case JIS_LOCALE_QUERY:
	case JIS_LOCALE_DEFAULT:
		break;
	}
	abort();
}
